"""Local Plugin."""
import decimal
import logging
import ast
import numpy as np

try:
    from urllib import parse  # Python 3
except ImportError:
    import urlparse as parse

from qtpy.QtCore import Slot, Qt
from pydm.data_plugins.plugin import PyDMPlugin, PyDMConnection

logger = logging.getLogger(__name__)


class Connection(PyDMConnection):
    def __init__(self, channel, address, protocol=None, parent=None):
        self._is_connection_configured = False
        self._value_type = None
        self._precision_set = None
        self._type_kwargs = {}

        self._required_config_keys = [
            'name',
            'type',
            'init'
        ]

        self._extra_config_keys = [
            "precision",
            "unit",
            "upper_limit",
            "lower_limit",
            "enum_string"
        ]

        self._extra_numpy_config_keys = [
            "dtype",
            "copy",
            "order",
            "subok",
            "ndmin"
        ]

        self._data_types = {
            'int': int,
            'float': float,
            'str': str,
            'bool': bool,
            'array': np.array
        }

        self._precision = None
        self._unit = None
        self._upper_limit = None
        self._lower_limit = None
        self._enum_string = None

        super(Connection, self).__init__(channel, address, protocol, parent)
        self._configuration = {}
        self.add_listener(channel)
        self.send_connection_state(False)
        self.send_access_state()
        self.connected = False

    def _configure_local_plugin(self, channel):
        if self._is_connection_configured:
            logger.debug('LocalPlugin connection already configured.')
            return

        try:
            url_data = UrlToPython(channel).get_info()
        except ValueError("Not enough information"):
            logger.debug('Invalid configuration for LocalPlugin connection', exc_info=True)
            return

        self._configuration['name'] = url_data[1]
        address = url_data[2]

        if url_data[0] is None:
            return

        self._configuration.update(url_data[0])
        self.address = address

        # set the object's attributes
        init_value = self._configuration.get('init')[0]
        self._value_type = self._configuration.get('type')[0]
        self.name = self._configuration.get('name')

        # get the extra info if any
        self.parse_channel_extras(self._configuration)

        # send initial values
        self.value = self.convert_value(init_value, self._value_type)
        self.connected = True
        self.send_connection_state(True)
        self.send_new_value(self.value)

        # set connection configured to true
        self._is_connection_configured = True

    def parse_channel_extras(self, extras):
        """
        Parse the extras dictionary and either pass the data
        to the appropriate methods that will take care of it,
        or emit it right away.

        Parameters
        ----------
        extras : dict
            Dictionary containing extra parameters for the
            this local variable configuration
            These parameters are optional

        Returns
        -------
        None.

        """

        precision = extras.get('precision')
        if precision is not None:
            try:
                self._precision_set = int(precision[0])
                self.prec_signal.emit(self._precision_set)
            except ValueError:
                logger.debug('Cannot convert precision value=%r', precision)
        unit = extras.get('unit')
        if unit is not None:
            self.unit_signal.emit(str(unit[0]))
        upper_limit = extras.get('upper_limit')
        if upper_limit is not None and (self._value_type == 'float' or self._value_type == 'int'):
            self.send_upper_limit(upper_limit[0])
        lower_limit = extras.get('lower_limit')
        if lower_limit is not None and (self._value_type == 'float' or self._value_type == 'int'):
            self.send_lower_limit(lower_limit[0])
        enum_string = extras.get('enum_string')
        if enum_string is not None:
            self.send_enum_string(enum_string[0])

        type_kwargs = {k: v for k, v in extras.items()
                       if k in self._extra_numpy_config_keys}

        if type_kwargs:
            self.format_type_params(type_kwargs)

        unused = {k: v for k, v in extras.items()
                       if k not in self._extra_config_keys
                       and k not in self._required_config_keys
                       and k not in self._extra_numpy_config_keys}

        if len(unused) == 0:
            return

        info = "The following entries are not valid config keys:"
        for items in unused:
            info = info + " " + items + ","

        logger.debug(info)

    @Slot(int)
    @Slot(float)
    @Slot(str)
    @Slot(bool)
    @Slot(np.ndarray)
    def send_new_value(self, value):
        """
        Send the values sent through a specific local
        variable channel to all its listeners.
        """

        if value is not None:
            self.new_value_signal[type(value)].emit(value)

    def send_precision(self, value):
        """
        Calculate and send the precision for float values if precision
        is not specified in the extras

        Parameters
        ----------
        value : int
            The value to be sent,should be sent as int

        Returns
        -------
        None.

        """
        if value is not None:
            self._precision = self.precision_for_value(value)
            self.prec_signal.emit(self._precision)

    def send_unit(self, unit):
        """
        Send the unit for the data.

        Parameters
        ----------
        unit : string

        Returns
        -------
        None.

        """
        if unit is not None:
            self._unit = str(unit)
            self.prec_signal.emit(self.unit)

    def send_upper_limit(self, upper_limit):
        """
        Send the upper limit value as float or int.

        Parameters
        ----------
        upper_limit : int or float

        Returns
        -------
        None.

        """
        if upper_limit is not None:
            if self._value_type == "int":
                self._upper_limit = int(upper_limit)
            else:
                self._upper_limit = float(upper_limit)

            self.upper_ctrl_limit_signal.emit(self._upper_limit)

    def send_lower_limit(self, lower_limit):
        """
        Send the lower limit value as float or int.

        Parameters
        ----------
        lower_limit : int, float, or str

        Returns
        -------
        None.

        """
        if lower_limit is not None:
            if self._value_type == "int":
                self._lower_limit = int(lower_limit)
            else:
                self._lower_limit = float(lower_limit)

            self.lower_ctrl_limit_signal.emit(self._lower_limit)

    def send_enum_string(self, enum_string):
        """
        Send enum_string as tuple of strings.

        Parameters
        ----------
        enum_string : int or float

        Returns
        -------
        None.

        """
        if enum_string is not None:
            try:
                self._enum_string = tuple(ast.literal_eval(enum_string))
                self.enum_strings_signal.emit(self._enum_string)
            except ValueError:
                logger.debug("Error when converting enum_string")

    def format_type_params(self, type_kwargs):
        """
        Format value_type parameters.

        Parameters
        ----------
        type_kwargs : dict

        Returns
        -------
            dict

        """

        dtype = type_kwargs.get('dtype')

        if dtype is not None:
            dtype = dtype[0]
            try:
                self._type_kwargs['dtype'] = np.dtype(dtype)
                return self._type_kwargs
            except TypeError:
                logger.debug('Cannot convert dtype value=%r', dtype)
        else:
            dtype = 'object'
            try:
                self._type_kwargs['dtype'] = np.dtype(dtype)
                return self._type_kwargs
            except TypeError:
                logger.debug('Cannot convert dtype value=%r', dtype)

        return self._type_kwargs

    def send_access_state(self):
        """
        Send True for all the widgets using Local Plugin.

        Returns
        -------
        None.

        """
        self.write_access_signal.emit(True)

    def convert_value(self, value, value_type):
        """
        Convert values to their appropriate types.

        Parameters
        ----------
        value :
            Data for this variable.
        value_type : str
            Data type intended for this variable.

        Returns
        -------
            The data for this variable converted to its appropriate type

        """

        _type = self._data_types.get(value_type)
        if _type is not None:
            try:
                if value_type == "array":
                    value = ast.literal_eval(value)
                return _type(value, **self._type_kwargs)
            except ValueError:
                logger.debug('Cannot convert value_type')
        else:
            return None

    def send_connection_state(self, conn):
        self.connected = conn
        self.connection_state_signal.emit(conn)

    def add_listener(self, channel):
        super(Connection, self).add_listener(channel)
        self._configure_local_plugin(channel)
        # send write access == True to the listeners
        self.send_access_state()
        # send new values to the listeners right away
        self.send_new_value(self.value)
        # send the precision in case of float values
        if isinstance(self.value, float):
            if self._precision_set is None:
                self.send_precision(self.value)
            else:
                self.prec_signal.emit(self._precision_set)

        if channel.connection_slot is not None:
            self.send_connection_state(conn=True)

        # Connect the put_value slot to the channel's value_signal,
        # which captures the values sent through the plugin
        if channel.value_signal is not None:
            try:
                channel.value_signal[int].connect(
                    self.put_value, Qt.QueuedConnection)
            except KeyError:
                pass
            try:
                channel.value_signal[float].connect(
                    self.put_value, Qt.QueuedConnection)
            except KeyError:
                pass
            try:
                channel.value_signal[str].connect(
                    self.put_value, Qt.QueuedConnection)
            except KeyError:
                pass
            try:
                channel.value_signal[bool].connect(
                    self.put_value, Qt.QueuedConnection)
            except KeyError:
                pass
            try:
                channel.value_signal[np.ndarray].connect(
                    self.put_value, Qt.QueuedConnection)
            except KeyError:
                pass

    @Slot(int)
    @Slot(float)
    @Slot(str)
    @Slot(bool)
    @Slot(np.ndarray)
    def put_value(self, new_value):
        """
        Slot connected to the channel.value_signal.
        Updates the value of this local variable and then broadcasts it to
        the other listeners to this channel
        """
        if new_value is not None:
            # update the attributes here with the new values
            self.value = new_value
            # send this value
            self.send_new_value(new_value)
            # send precision for float values
            if isinstance(new_value, float):
                if self._precision_set is None:
                    self.send_precision(new_value)
                else:
                    self.prec_signal.emit(self._precision_set)

    @staticmethod
    def precision_for_value(value, max_precision=8):
        dec = decimal.Decimal(str(value))
        solution = min((len(str(dec).split('.')[1]), max_precision))
        return solution


class LocalPlugin(PyDMPlugin):
    protocol = "loc"
    connection_class = Connection

    @staticmethod
    def get_connection_id(channel):
        obj = UrlToPython(channel)
        name = obj.get_info()[1]
        return name


class UrlToPython:
    def __init__(self, channel):
        self.channel = channel

    def get_info(self):
        """
        Parses a given url into a list and a string.

        Parameters
        ----------

        Returns
        -------
        A tuple: (<list>, <str>)
        """
        address = PyDMPlugin.get_address(self.channel)
        address = "loc://" + address
        name = None
        config = None

        try:
            config = parse.parse_qs(parse.urlsplit(address).query)
            name = parse.urlsplit(address).netloc

            if not name or not config:
                raise
            elif config['init'] is None or config['type'] is None:
                raise
        except Exception:
            try:
                if not name:
                    raise
                logger.debug('LocalPlugin connection %s got new listener.', address)
                return None, name, address
            except Exception:
                msg = "Invalid configuration for LocalPlugin connection. %s"
                logger.exception(msg, address, exc_info=True)
                raise ValueError("error in local data plugin input")

        return config, name, address



import os
import json
import copy
import pytest
import webbrowser

from qtpy import QtCore
from qtpy.QtWidgets import QMessageBox, QTableWidgetSelectionRange

from ...widgets.base import PyDMPrimitiveWidget
from ...widgets.rules_editor import RulesEditor


class DummyWidget:
    """
    A stub class to play with the rules editor and not touch
    the RulesEngine
    """
    RULE_PROPERTIES = PyDMPrimitiveWidget.RULE_PROPERTIES
    DEFAULT_RULE_PROPERTY = PyDMPrimitiveWidget.DEFAULT_RULE_PROPERTY

    def __init__(self, *args, **kwargs):
        self._rules = None

    @property
    def rules(self):
        return self._rules

    @rules.setter
    def rules(self, new_rules):
        if new_rules != self._rules:
            self._rules = new_rules


@pytest.mark.parametrize(
    "use_enum, visible",
    [
        (None, False),
        (True, True),
        (False, False)
    ]
)
def test_rules_editor(use_enum, visible, qtbot, monkeypatch):
    """
    Test the rules editor in general.

    Parameters
    ----------
    qtbot : fixture
        pytest-qt window for widget test
    monkeypatch : fixture
        To override dialog behaviors
    """

    # Create the base widget
    widget = DummyWidget()

    # Ensure that no rules are set
    assert widget.rules is None

    # Create a rule editor for this widget
    empty = RulesEditor(widget)
    qtbot.addWidget(empty)

    empty.show()

    qtbot.waitExposed(empty, timeout=5000)

    # Abort the changes
    empty.cancelChanges()

    # Create the rules data for the widget
    rules_list = [{'name': 'Rule #1', 'property': 'Enable',
                   'initial_value': 'False',
                   'expression': 'ch[0] > 1',
                   'channels': [
                       {'channel': 'ca://MTEST:Float', 'trigger': True}]}]
    if use_enum is not None:
        rules_list[0]["channels"][0]["use_enum"] = use_enum

    # Add the rules to the widget
    widget.rules = json.dumps(rules_list)

    # Create a new Editor Window
    re = RulesEditor(widget)
    qtbot.addWidget(re)
    re.show()

    qtbot.waitExposed(re, timeout=5000)

    assert re.lst_rules.count() == 1
    assert not re.frm_edit.isEnabled()

    ch_choices = {True: QtCore.Qt.Checked, False: QtCore.Qt.Unchecked}

    re.lst_rules.setCurrentRow(0)
    re.load_from_list()
    assert re.frm_edit.isEnabled()
    assert re.txt_name.text() == 'Rule #1'
    assert re.cmb_property.currentText() == 'Enable'
    assert re.tbl_channels.rowCount() == 1
    assert re.tbl_channels.item(0, 0).text() == 'ca://MTEST:Float'
    assert re.tbl_channels.item(0, 1).checkState() == QtCore.Qt.Checked
    assert re.tbl_channels.item(0, 2).checkState() == ch_choices[visible]
    assert re.lbl_expected_type.text() == 'bool'
    assert re.txt_expression.text() == 'ch[0] > 1'
    assert re.txt_initial_value.text() == 'False'

    qtbot.keyClicks(re.txt_name, '-Test')
    qtbot.keyClick(re.txt_name, QtCore.Qt.Key_Return)
    assert re.txt_name.text() == 'Rule #1-Test'
    assert re.rules[0]['name'] == 'Rule #1-Test'

    qtbot.mouseClick(re.btn_add_channel, QtCore.Qt.LeftButton)
    re.tbl_channels.item(1, 0).setText("ca://TEST")
    assert re.rules[0]['channels'][1]['channel'] == 'ca://TEST'
    assert re.rules[0]['channels'][1]['trigger'] is False
    assert re.rules[0]['channels'][1]['use_enum'] is True

    re.txt_expression.clear()
    qtbot.keyClicks(re.txt_expression, 'ch[0] < 1')
    qtbot.keyClick(re.txt_expression, QtCore.Qt.Key_Return)
    assert re.txt_expression.text() == 'ch[0] < 1'
    assert re.rules[0]['expression'] == 'ch[0] < 1'

    re.txt_initial_value.clear()
    qtbot.keyClicks(re.txt_initial_value, 'True')
    qtbot.keyClick(re.txt_initial_value, QtCore.Qt.Key_Return)
    assert re.txt_initial_value.text() == 'True'
    assert re.rules[0]['initial_value'] == 'True'

    # Test Delete with Confirm - NO
    assert re.tbl_channels.rowCount() == 2
    re.tbl_channels.setRangeSelected(QTableWidgetSelectionRange(1, 0, 1, 1), True)
    monkeypatch.setattr(QMessageBox, 'question', lambda *args: QMessageBox.No)
    qtbot.mouseClick(re.btn_del_channel, QtCore.Qt.LeftButton)
    assert re.tbl_channels.rowCount() == 2

    # Test Delete with Confirm - YES
    re.tbl_channels.setRangeSelected(QTableWidgetSelectionRange(1, 0, 1, 1), True)
    monkeypatch.setattr(QMessageBox, 'question', lambda *args: QMessageBox.Yes)
    qtbot.mouseClick(re.btn_del_channel, QtCore.Qt.LeftButton)
    assert re.tbl_channels.rowCount() == 1
    assert len(re.rules[0]['channels']) == 1

    # Test Delete with Invalid Selection
    re.tbl_channels.setRangeSelected(QTableWidgetSelectionRange(1, 0, 1, 1), True)
    monkeypatch.setattr(QMessageBox, 'question', lambda *args: QMessageBox.Yes)
    qtbot.mouseClick(re.btn_del_channel, QtCore.Qt.LeftButton)
    assert re.tbl_channels.rowCount() == 1
    assert len(re.rules[0]['channels']) == 1

    qtbot.mouseClick(re.btn_add_rule, QtCore.Qt.LeftButton)
    assert re.lst_rules.count() == 2
    assert re.frm_edit.isEnabled()
    assert re.txt_name.text() == 'New Rule'
    assert re.cmb_property.currentText() == widget.DEFAULT_RULE_PROPERTY
    assert re.tbl_channels.rowCount() == 0
    assert re.txt_expression.text() == ''

    qtbot.mouseClick(re.btn_add_channel, QtCore.Qt.LeftButton)
    assert re.tbl_channels.item(0, 0).text() == ''
    assert re.tbl_channels.item(0, 1).checkState() == QtCore.Qt.Checked
    assert re.tbl_channels.item(0, 2).checkState() == QtCore.Qt.Checked

    qtbot.mouseClick(re.btn_add_channel, QtCore.Qt.LeftButton)
    assert re.tbl_channels.item(1, 0).text() == ''
    assert re.tbl_channels.item(1, 1).checkState() == QtCore.Qt.Unchecked
    assert re.tbl_channels.item(1, 2).checkState() == QtCore.Qt.Checked

    # Switch between the rules
    re.lst_rules.setCurrentRow(0)
    re.lst_rules.setCurrentRow(1)

    # Delete Rule 1 - Confirm - NO
    monkeypatch.setattr(QMessageBox, 'question', lambda *args: QMessageBox.No)
    qtbot.mouseClick(re.btn_del_rule, QtCore.Qt.LeftButton)
    assert re.lst_rules.count() == 2

    # Delete Rule 1 - Confirm - YES
    monkeypatch.setattr(QMessageBox, 'question', lambda *args: QMessageBox.Yes)
    qtbot.mouseClick(re.btn_del_rule, QtCore.Qt.LeftButton)
    assert re.frm_edit.isEnabled() is False
    assert re.lst_rules.count() == 1

    re.lst_rules.setCurrentRow(0)
    monkeypatch.setattr(QMessageBox, 'question', lambda *args: QMessageBox.Yes)
    qtbot.mouseClick(re.btn_del_rule, QtCore.Qt.LeftButton)
    assert re.frm_edit.isEnabled() is False
    assert re.lst_rules.count() == 0

    # Delete Empty List - Confirm - YES
    monkeypatch.setattr(QMessageBox, 'question', lambda *args: QMessageBox.Yes)
    qtbot.mouseClick(re.btn_del_rule, QtCore.Qt.LeftButton)
    assert re.frm_edit.isEnabled() is False
    assert re.lst_rules.count() == 0
    re.cancelChanges()


def test_rules_editor_data_valid(qtbot):
    """
    Test the rules form validation.

    Parameters
    ----------
    qtbot : fixture
        pytest-qt window for widget test
    """
    def validate(expected_status, expected_msg, definition):
        status, msg = RulesEditor.is_data_valid(definition)
        assert status == expected_status
        assert expected_msg in msg

    rules_list = [{'name': 'Rule #1', 'property': 'Enable',
                   'expression': 'ch[0] > 1',
                   'channels': [
                       {'channel': 'ca://MTEST:Float', 'trigger': True}]}]

    validate(True, '', rules_list)

    rules_original = copy.deepcopy(rules_list)

    rules_original[0]['name'] = ''
    validate(False, 'has no name', rules_original)

    rules_original = copy.deepcopy(rules_list)
    rules_original[0]['expression'] = ''
    validate(False, 'has no expression', rules_original)

    rules_original = copy.deepcopy(rules_list)
    old_channels = rules_original[0]['channels']
    rules_original[0]['channels'] = []
    validate(False, 'has no channel', rules_original)
    rules_original[0]['channels'] = old_channels

    rules_original[0]['channels'][0]['trigger'] = False
    validate(False, 'has no channel for trigger', rules_original)

    rules_original[0]['channels'][0]['channel'] = None
    validate(False, 'Ch. #0 has no channel.', rules_original)
    rules_original[0]['channels'][0]['channel'] = ''
    validate(False, 'Ch. #0 has no channel.', rules_original)


def test_rules_editor_open_help(qtbot, monkeypatch):
    """
    Test the Open Help button

    Parameters
    ----------
    qtbot : fixture
        pytest-qt window for widget test
    monkeypatch : fixture
        To override dialog behaviors
    """
    # Create the base widget
    widget = DummyWidget()

    # Create a new Editor Window
    re = RulesEditor(widget)
    qtbot.addWidget(re)
    re.show()

    qtbot.waitExposed(re, timeout=5000)

    re.lst_rules.setCurrentRow(0)

    url = re.open_help(open=False)
    base_url = os.getenv("PYDM_DOCS_URL", "https://slaclab.github.io/pydm")
    exp_url = base_url+"/widgets/widget_rules/index.html"
    assert url == exp_url

    monkeypatch.setattr(webbrowser, 'open',
                        lambda *args, **kwargs: '')
    re.open_help()
    re.cancelChanges()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'about.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from qtpy import QtCore, QtWidgets, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(474, 288)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(Form)
        self.tabWidget.setMinimumSize(QtCore.QSize(450, 0))
        self.tabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidget.setObjectName("tabWidget")
        self.versionTab = QtWidgets.QWidget()
        self.versionTab.setObjectName("versionTab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.versionTab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.pydmLabel = QtWidgets.QLabel(self.versionTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pydmLabel.sizePolicy().hasHeightForWidth())
        self.pydmLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(24)
        font.setBold(True)
        font.setWeight(75)
        self.pydmLabel.setFont(font)
        self.pydmLabel.setTextFormat(QtCore.Qt.PlainText)
        self.pydmLabel.setScaledContents(False)
        self.pydmLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.pydmLabel.setObjectName("pydmLabel")
        self.verticalLayout_2.addWidget(self.pydmLabel)
        self.pydmVersionLabel = QtWidgets.QLabel(self.versionTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pydmVersionLabel.sizePolicy().hasHeightForWidth())
        self.pydmVersionLabel.setSizePolicy(sizePolicy)
        self.pydmVersionLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.pydmVersionLabel.setObjectName("pydmVersionLabel")
        self.verticalLayout_2.addWidget(self.pydmVersionLabel)
        self.modulesVersionLabel = QtWidgets.QLabel(self.versionTab)
        self.modulesVersionLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.modulesVersionLabel.setWordWrap(True)
        self.modulesVersionLabel.setObjectName("modulesVersionLabel")
        self.verticalLayout_2.addWidget(self.modulesVersionLabel)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.tabWidget.addTab(self.versionTab, "")
        self.externalToolsTab = QtWidgets.QWidget()
        self.externalToolsTab.setObjectName("externalToolsTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.externalToolsTab)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.externalToolsTableWidget = QtWidgets.QTableWidget(self.externalToolsTab)
        self.externalToolsTableWidget.setObjectName("externalToolsTableWidget")
        self.externalToolsTableWidget.setColumnCount(0)
        self.externalToolsTableWidget.setRowCount(0)
        self.verticalLayout_3.addWidget(self.externalToolsTableWidget)
        self.tabWidget.addTab(self.externalToolsTab, "")
        self.dataPluginsTab = QtWidgets.QWidget()
        self.dataPluginsTab.setObjectName("dataPluginsTab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.dataPluginsTab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.dataPluginsTableWidget = QtWidgets.QTableWidget(self.dataPluginsTab)
        self.dataPluginsTableWidget.setObjectName("dataPluginsTableWidget")
        self.dataPluginsTableWidget.setColumnCount(0)
        self.dataPluginsTableWidget.setRowCount(0)
        self.verticalLayout_4.addWidget(self.dataPluginsTableWidget)
        self.tabWidget.addTab(self.dataPluginsTab, "")
        self.contributorsTab = QtWidgets.QWidget()
        self.contributorsTab.setObjectName("contributorsTab")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.contributorsTab)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.contributorsListWidget = QtWidgets.QListWidget(self.contributorsTab)
        self.contributorsListWidget.setObjectName("contributorsListWidget")
        self.verticalLayout_5.addWidget(self.contributorsListWidget)
        self.tabWidget.addTab(self.contributorsTab, "")
        self.verticalLayout.addWidget(self.tabWidget)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(0)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "About PyDM"))
        self.pydmLabel.setText(_translate("Form", "PyDM"))
        self.pydmVersionLabel.setText(_translate("Form", "Version {version}"))
        self.modulesVersionLabel.setText(_translate("Form", "Using Python v{pyver}, Numpy v{numpyver}, PyQt v{pyqtver}, Qt v{qtver}, and PyQtGraph v{pyqtgraphver}."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.versionTab), _translate("Form", "Version"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.externalToolsTab), _translate("Form", "External Tools"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.dataPluginsTab), _translate("Form", "Data Plugins"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.contributorsTab), _translate("Form", "Contributors"))




from qtpy.QtWidgets import QCheckBox
from .base import PyDMWritableWidget


class PyDMCheckbox(QCheckBox, PyDMWritableWidget):
    """
    A QCheckbox with support for Channels and more from PyDM

    Parameters
    ----------
    parent : QWidget
        The parent widget for the Label
    init_channel : str, optional
        The channel to be used by the widget.

    """
    def __init__(self, parent=None, init_channel=None):
        QCheckBox.__init__(self, parent)
        PyDMWritableWidget.__init__(self, init_channel=init_channel)
        self.clicked.connect(self.send_value)

    def value_changed(self, new_val):
        """
        Callback invoked when the Channel value is changed.
        Sets the checkbox checked or not based on the new value.

        Parameters
        ----------
        new_val : int
            The new value from the channel.
        """
        super(PyDMCheckbox, self).value_changed(new_val)
        if new_val is None:
            return
        if new_val > 0:
            self.setChecked(True)
        else:
            self.setChecked(False)

    def send_value(self, checked):
        """
        Method that emit the signal to notify the Channel that a new
        value was written at the widget.

        Parameters
        ----------
        checked : bool
            True in case the checkbox was checked, False otherwise.
        """
        if checked:
            self.send_value_signal.emit(1)
        else:
            self.send_value_signal.emit(0)



import os

__all__ = ['DEFAULT_PROTOCOL',
           'DESIGNER_ONLINE',
           'STYLESHEET',
           'STYLESHEET_INCLUDE_DEFAULT',
           'CONFIRM_QUIT'
           ]


DEFAULT_PROTOCOL = os.getenv("PYDM_DEFAULT_PROTOCOL")
if DEFAULT_PROTOCOL is not None:
    # Get rid of the "://" part if it exists
    DEFAULT_PROTOCOL = DEFAULT_PROTOCOL.split("://")[0]

DESIGNER_ONLINE = os.getenv("PYDM_DESIGNER_ONLINE", None) is not None

STYLESHEET = os.getenv("PYDM_STYLESHEET", None)

STYLESHEET_INCLUDE_DEFAULT = os.getenv("PYDM_STYLESHEET_INCLUDE_DEFAULT", False)

CONFIRM_QUIT = os.getenv("PYDM_CONFIRM_QUIT", "n").lower() in ("y", "t", "1", "true")